==> metlo-0.1.2.tgz: package/src/middlewares/koa.ts <==
"use strict"
import { sendMetloTrace, shouldBlock } from "../rust_binds"
import { METLO_META } from "."
import Application from "koa"
import { MetloConfig } from "../types"
import { Logger } from "../utils/logger"

declare module "koa" {
  interface Request extends Application.BaseRequest {
    body?: any
  }
}

export interface MetloConfigKoa<
  _ctx extends Application.ExtendableContext = Application.ExtendableContext,
> extends MetloConfig {
  getUser?: (ctx: _ctx) => string
  rejectResponse?: (ctx: _ctx) => void
}

function versionCheck() {
  return true
}

const initialize = <
  _ctxType extends Application.ExtendableContext = Application.ExtendableContext,
>() => {
  if (!versionCheck()) {
    return
  }

  function compileRequest(ctx: Application.ExtendableContext) {
    return {
      url: {
        host: (
          METLO_META.opts?.apiHost || ctx.request.socket.remoteAddress
        ).toString(),
        path: ctx.path || ctx.originalUrl || "",
        parameters: Object.entries(ctx.query || {}).map(([k, v]) => ({
          name: k.toString(),
          value: v.toString(),
        })),
      },
      headers: Object.entries(ctx.request.headers || {}).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof ctx.request.body === "object"
        ? JSON.stringify(ctx.request.body)
        : ctx.request.body || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
      method: ctx.request.method.toString(),
    }
  }

  function compileMeta(ctx: Application.ExtendableContext) {
    return {
      environment: process.env.NODE_ENV || "development",
      incoming: true,
      source: ctx.request.socket.remoteAddress.toString(),
      sourcePort: ctx.request.socket.remotePort,
      destination: ctx.request.socket.localAddress.toString(),
      destinationPort: ctx.request.socket.localPort,
      metloSource: "node/koa",
    }
  }

  function compileResponse(ctx: Application.ExtendableContext) {
    return {
      status: ctx.response.status || 200,
      headers: Object.entries(ctx.response.headers || {}).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof ctx.body === "object"
        ? JSON.stringify(ctx.body)
        : ctx.body || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
    }
  }

  function compileInformation(ctx) {
    const data = JSON.stringify({
      request: compileRequest(ctx),
      response: compileResponse(ctx),
      meta: compileMeta(ctx),
    })

    let user = METLO_META.getUser
      ? (METLO_META as MetloConfigKoa<_ctxType>).getUser(ctx)
      : null

    sendMetloTrace(data, user)
  }

  const middleware: Application.Middleware = async (ctx, next) => {
    let rejectedResponse = false
    try {
      if (!METLO_META.disableBlocking) {
        const willBlock = shouldBlock(
          JSON.stringify(compileRequest(ctx)),
          JSON.stringify(compileMeta(ctx)),
        )
        if (willBlock) {
          rejectedResponse = true
          if (METLO_META.rejectResponse) {
            // @ts-ignore
            ;(METLO_META as MetloConfigKoa<_ctxType>).rejectResponse(ctx)
          } else {
            ctx.status = 403
          }
        }
      }
    } catch (err) {
      Logger().log("debug", "Metlo encountered an error")
      Logger().log("debug", err)
    }
    if (rejectedResponse) {
      // Don't do next if we're rejecting the response. 
      compileInformation(ctx)
    } else {
      try {
        await next()
        compileInformation(ctx)
      } catch (err) {
        compileInformation({
          ...ctx,
          response: {
            ...ctx.response,
            status: 500,
          },
          body: ctx.response.body
            ? ctx.response.body
            : err
            ? err.toString()
            : "",
        })
        throw err
      }
    }
  }
  return middleware
}

export const dummyModule = (): Application.Middleware => {
  return (ctx, next) => {
    next()
  }
}

export default initialize

==> metlo-0.1.3.tgz: package/src/middlewares/koa.ts <==
"use strict"
import { sendMetloTrace, shouldBlock } from "../rust_binds"
import { METLO_META } from "."
import Application from "koa"
import { MetloConfig } from "../types"
import { Logger } from "../utils/logger"

declare module "koa" {
  interface Request extends Application.BaseRequest {
    body?: any
  }
}

export interface MetloConfigKoa<
  _ctx extends Application.ExtendableContext = Application.ExtendableContext,
> extends MetloConfig {
  getUser?: (ctx: _ctx) => string
  rejectResponse?: (ctx: _ctx) => void
}

function versionCheck() {
  return true
}

const initialize = <
  _ctxType extends Application.ExtendableContext = Application.ExtendableContext,
>() => {
  if (!versionCheck()) {
    return
  }

  function compileRequest(ctx: Application.ExtendableContext) {
    return {
      url: {
        host: (
          METLO_META.opts?.apiHost || ctx.request.socket.remoteAddress
        ).toString(),
        path: ctx.path || ctx.originalUrl || "",
        parameters: Object.entries(ctx.query || {}).map(([k, v]) => ({
          name: k.toString(),
          value: v.toString(),
        })),
      },
      headers: Object.entries(ctx.request.headers || {}).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof ctx.request.body === "object"
        ? JSON.stringify(ctx.request.body)
        : ctx.request.body || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
      method: ctx.request.method.toString(),
      user: METLO_META.getUser
        ? (METLO_META as MetloConfigKoa).getUser(ctx)
        : null,
    }
  }

  function compileMeta(ctx: Application.ExtendableContext) {
    return {
      environment: process.env.NODE_ENV || "development",
      incoming: true,
      source: ctx.request.socket.remoteAddress.toString(),
      sourcePort: ctx.request.socket.remotePort,
      destination: ctx.request.socket.localAddress.toString(),
      destinationPort: ctx.request.socket.localPort,
      metloSource: "node/koa",
    }
  }

  function compileResponse(ctx: Application.ExtendableContext) {
    return {
      status: ctx.response.status || 200,
      headers: Object.entries(ctx.response.headers || {}).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof ctx.body === "object"
        ? JSON.stringify(ctx.body)
        : ctx.body || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
    }
  }

  function compileInformation(ctx) {
    const data = JSON.stringify({
      request: compileRequest(ctx),
      response: compileResponse(ctx),
      meta: compileMeta(ctx),
    })

    sendMetloTrace(data)
  }

  if (METLO_META.metloEnabled) {
    const middleware: Application.Middleware = async (ctx, next) => {
      let rejectedResponse = false
      try {
        if (!METLO_META.disableBlocking) {
          const willBlock = shouldBlock(
            JSON.stringify(compileRequest(ctx)),
            JSON.stringify(compileMeta(ctx)),
          )
          if (willBlock) {
            rejectedResponse = true
            if (METLO_META.rejectResponse) {
              // @ts-ignore
              ;(METLO_META as MetloConfigKoa<_ctxType>).rejectResponse(ctx)
            } else {
              ctx.status = 403
            }
          }
        }
      } catch (err) {
        Logger().log("debug", "Metlo encountered an error")
        Logger().log("debug", err)
      }
      if (rejectedResponse) {
        // Don't do next if we're rejecting the response.
        compileInformation(ctx)
      } else {
        try {
          await next()
          compileInformation(ctx)
        } catch (err) {
          compileInformation({
            ...ctx,
            response: {
              ...ctx.response,
              status: 500,
            },
            body: ctx.response.body
              ? ctx.response.body
              : err
              ? err.toString()
              : "",
          })
          throw err
        }
      }
    }
    return middleware
  } else {
    return dummyModule()
  }
}

export const dummyModule = (): Application.Middleware => {
  return (ctx, next) => {
    next()
  }
}

export default initialize
